@@ -126,37 +126,36 @@ sub applyClause {
     print STDERR "Rewrite replace at " . $tree->toString . " using $pattern\n"
       if $LaTeXML::Core::Rewrite::DEBUG;
     my $parent = $tree->parentNode;
-
     # Remove & separate nodes to be replaced, and sibling nodes following them.
     my @following = ();    # Collect the matching and following nodes
     while (my $sib = $parent->lastChild) {
       $parent->removeChild($sib);
       unshift(@following, $sib);
-      last if $$sib == $$tree; }
+      last if $tree->isSameNode($sib); }
     my @replaced = map { shift(@following) } 1 .. $n_to_replace;    # Remove the nodes to be replaced
-
+    map { $document->unRecordNodeIDs($_) } @replaced;
     # Carry out the operation, inserting whatever nodes.
     $document->setNode($parent);
     my $point = $parent->lastChild;
     &$pattern($document, @replaced);                                # Carry out the insertion.
 
-    # Now collect the newly inserted nodes and store in a _Capture_ node.
+    # Now collect the newly inserted nodes for any needed patching
     my @inserted = ();                                              # Collect the newly added nodes.
     if ($point) {
-      while (my $sib = $parent->lastChild) {
-        $parent->removeChild($sib);
-        unshift(@inserted, $sib);
-        last if $$sib == $$point; } }
+      my @sibs = $parent->childNodes;
+      while (my $sib = pop(@sibs)) {
+        last if $$sib == $$point;
+        unshift(@inserted, $sib); } }
     else {
       @inserted = $parent->childNodes; }
-    my $insertion = $document->openElement('_Capture_', font => $document->getNodeFont($parent));
-    map { $insertion->appendChild($_) } @inserted;
 
-    # Now remove the insertion and replace with rewritten nodes and replace the following siblings.
-    @inserted = $insertion->childNodes;
-    $parent->removeChild($insertion);
-    map { $parent->appendChild($_) } @inserted, @following;
-  }
+    # Now make any adjustments to the new nodes
+    map { $document->recordNodeIDs($_) } @inserted;
+    my $font = $document->getNodeFont($tree);                       # the font of the matched node
+    foreach my $ins (@inserted) {    # Copy the non-semantic parts of font to the replacement
+      $document->mergeNodeFontRec($ins => $font); }
+    # Now, replace the following nodes.
+    map { $parent->appendChild($_) } @following; }
   elsif ($op eq 'action') {
     print STDERR "Rewrite action at " . $tree->toString . " using $pattern\n"
       if $LaTeXML::Core::Rewrite::DEBUG;
@@ -200,8 +199,9 @@ sub compileClause {
       $pattern = ["descendant-or-self::*[\@xml:id='" . $self->getLabelID($1) . "']", 1]; }
     elsif ($pattern =~ /^id:(.*)$/) {
       $pattern = ["descendant-or-self::*[\@xml:id='$1']", 1]; }
-    elsif ($pattern =~ /^(.*):(.*)$/) {
-      $pattern = ["descendant-or-self::*[local-name()='$1' and \@refnum='$2']", 1]; }
+### Is this pattern ever used? <elementname>:<refnum> expects attribute!!!
+###    elsif ($pattern =~ /^(.*):(.*)$/) {
+###      $pattern = ["descendant-or-self::*[local-name()='$1' and \@refnum='$2']", 1]; }
     else {
       Error('misdefined', '<rewrite>', undef,
         "Unrecognized scope pattern in Rewrite clause: \"$pattern\"; Ignoring it.");
@@ -256,12 +256,11 @@ sub compile_match1 {
   map { $frag->appendChild($_) } @nodes;
   # Convert the captured nodes to an XPath that would match them.
   my $xpath = domToXPath($capdocument, $frag);
-  # For math, restrict to NOT operate on presentation branch of XMDual.
-  # The semantics should already be associated with it, through the XMDual itself.
-  # This assumes that any arguments in the presentation branch are by reference (XMRef)
-  # to the same args in the content branch --- thus the args will still be matched.
-  $xpath .= "[not(ancestor-or-self::*[parent::ltx:XMDual and not(following-sibling::*)])]"
-    if $$self{math};
+  # The branches of an XMDual can contain "decorations", nodes that are ONLY visible
+  # from either presentation or content, but not both.
+  # [See LaTeXML::Core::Document->markXMNodeVisibility]
+  # These decorations should NOT have rewrite rules applied
+  $xpath .= '[@_pvis and @_cvis]' if $$self{math};
 
   print STDERR "Converting \"" . ToString($patternbox) . "\"\n  => xpath= \"$xpath\"\n"
     if $LaTeXML::Core::Rewrite::DEBUG;
@@ -314,7 +313,7 @@ sub domToXPath {
   return "descendant-or-self::" . domToXPath_rec($document, $node); }
 
 # May need some work here;
-my %EXCLUDED_MATCH_ATTRIBUTES = (scriptpos => 1, 'xml:id' => 1);    # [CONSTANT]
+my %EXCLUDED_MATCH_ATTRIBUTES = (scriptpos => 1, 'xml:id' => 1, fontsize => 1);    # [CONSTANT]
 
 sub domToXPath_rec {
   my ($document, $node, @extra_predicates) = @_;
