@@ -118,18 +118,21 @@ sub getGraphicsSourceTypes {
 # Return the pathname to an appropriate image.
 sub findGraphicFile {
   my ($self, $doc, $node) = @_;
-  if (my $name = $node->getAttribute('graphic')) {
+  if (my $source = $node->getAttribute('graphic')) {
     # Find all acceptable image files, in order of search paths
-    my @paths = pathname_findall($name, paths => $LaTeXML::Post::Graphics::SEARCHPATHS,
+    my ($dir, $name, $reqtype) = pathname_split($source);
+    # Ignore the requested type? Or should it increase desirability?
+    my $file = pathname_concat($dir, $name);
+    my @paths = pathname_findall($file, paths => $LaTeXML::Post::Graphics::SEARCHPATHS,
       # accept empty type, incase bad type name, but actual file's content is known type.
       types => ['', $self->getGraphicsSourceTypes]);
     my ($best, $bestpath) = (-1, undef);
     # Now, find the first image that is either the correct type,
     # or has the most desirable type mapping
     foreach my $path (@paths) {
-      my $type         = pathname_type($path);
-      my $props        = $$self{type_properties}{$type};
-      my $desirability = $$props{desirability} || ($type eq $$props{destination_type} ? 10 : 0);
+      my $type  = pathname_type($path);
+      my $props = $$self{type_properties}{$type};
+      my $desirability = $$props{desirability} || ($type eq ($$props{destination_type} || 'notype') ? 10 : 0);
       if ($desirability > $best) {
         $best     = $desirability;
         $bestpath = $path; } }
@@ -149,7 +152,7 @@ sub getTransform {
 sub getTypeProperties {
   my ($self, $source, $options) = @_;
   my ($dir,  $name,   $ext)     = pathname_split($source);
-  my $props = $$self{type_properties}{$ext};
+  my $props = $$self{type_properties}{ lc($ext) };
   if (!$props) {
     # If we don't have a known file type, try a bit harder (maybe less efficient)
     if (my $type = image_type($source)) {
@@ -161,15 +164,18 @@ sub getTypeProperties {
 sub setGraphicSrc {
   my ($self, $node, $src, $width, $height) = @_;
   $node->setAttribute('imagesrc',    $src);
-  $node->setAttribute('imagewidth',  $width);
-  $node->setAttribute('imageheight', $height);
+  $node->setAttribute('imagewidth',  $width) if defined $width;
+  $node->setAttribute('imageheight', $height) if defined $height;
   return; }
 
 sub processGraphic {
   my ($self, $doc, $node) = @_;
   my $source = $self->findGraphicFile($doc, $node);
   if (!$source) {
-    Warn('expected', 'source', $node, "No graphic source specified; skipping"); return; }
+    Warn('expected', 'source', $node, "No graphic source found; skipping",
+      "source was " . ($node->getAttribute('graphic') || 'none'));
+    $node->setAttribute('imagesrc', $node->getAttribute('graphic'));
+    return; }
   my $transform = $self->getTransform($node);
   my ($image, $width, $height) = $self->transformGraphic($doc, $node, $source, $transform);
   # $image should probably be relative already, except corner applications?
@@ -186,23 +192,25 @@ sub transformGraphic {
   ($sourcedir) = $doc->getSearchPaths unless $sourcedir;    # Fishing...
   my ($reldir, $name, $srctype)
     = pathname_split(pathname_relative($source, $sourcedir));
-  my $key = (ref $self) . ':' . join('|', "$reldir$name.$srctype",
+  my %properties = $self->getTypeProperties($source, $transform);
+  return Warn('unexpected', 'graphics_format', undef,
+    "Don't know what to do with graphics file format '$source'") unless %properties;
+  my $type = $properties{destination_type} || $srctype;
+  my $key = (ref $self) . ':' . join('|', "$reldir$name.$srctype.$type",
     map { join(' ', @$_) } @$transform);
   NoteProgressDetailed("\n[Processing $source as key=$key]");
 
-  my %properties = $self->getTypeProperties($source, $transform);
-  return Warn('unexpected', $source, undef,
-    "Don't know what to do with graphics file format '$source'") unless %properties;
-  my $type = $properties{destination_type} || $srctype;
   my $dest = $self->desiredResourcePathname($doc, $node, $source, $type);
   if (my $prev = $doc->cacheLookup($key)) {                 # Image was processed on previous run?
-    if ($prev =~ /^(.*?)\|(\d+)\|(\d+)$/) {
+    if ($prev =~ /^(.*?)\|(\d*)\|(\d*)$/) {
       my ($cached, $width, $height) = ($1, $2, $3);
+      $width  = undef unless $width;
+      $height = undef unless $height;
       # If so, check that it is still there, up to date, etc.
       if ((!defined $dest) || ($cached eq $dest)) {
         my $absdest = pathname_absolute($cached, $doc->getDestinationDirectory);
         if (pathname_timestamp($source) <= pathname_timestamp($absdest)) {
-          NoteProgressDetailed(" [Reuse $cached @ $width x $height]");
+          NoteProgressDetailed(" [Reuse $cached @ " . ($width || '?') . " x " . ($height || '?') . "]");
           return ($cached, $width, $height); } } } }
   # Trivial scaling case: Use original image with (at most) different width & height.
   my $triv_scaling = $$self{trivial_scaling} && ($type eq $srctype)
@@ -246,10 +254,18 @@ sub transformGraphic {
 
     NoteProgressDetailed(" [Destination $absdest]");
     ($width, $height) = image_graphicx_trivial($source, $transform, ddpt => $$self{ddpt});
-    return unless $width && $height;
+    if (!($width && $height)) {
+      if (!image_can_image()) {
+        Warn('imageprocessing', 'imagesize', undef,
+          "No image processing module found for image sizing",
+          "Will omit image size for $source",
+          "Please install one of: " . join(',', image_classes())); }
+      else {
+        Warn('expected', 'image', undef,
+          "Couldn't get usable image size for $source"); } }
     pathname_copy($source, $absdest)
       or Warn('I/O', $absdest, undef, "Couldn't copy $source to $absdest", "Response was: $!");
-    NoteProgressDetailed(" [Copied to $dest for $width x $height]"); }
+    NoteProgressDetailed(" [Copied to $dest @ " . ($width || '?') . " x " . ($height || '?') . "]"); }
   else {
     # With a complex transformation, we really needs a new name (well, don't we?)
     $dest = $self->generateResourcePathname($doc, $node, $source, $type) unless $dest;
@@ -257,11 +273,14 @@ sub transformGraphic {
     NoteProgressDetailed(" [Destination $absdest]");
     ($image, $width, $height) = image_graphicx_complex($source, $transform,
       ddpt => $$self{ddpt}, background => $$self{background}, %properties);
-    return unless $image && $width && $height;
+    if (!($image && $width && $height)) {
+      Warn('expected', 'image', undef,
+        "Couldn't get usable image for $source");
+      return; }
     NoteProgressDetailed(" [Writing to $absdest]");
     image_write($image, $absdest) or return; }
 
-  $doc->cacheStore($key, "$dest|$width|$height");
+  $doc->cacheStore($key, "$dest|" . ($width || '') . '|' . ($height || ''));
   NoteProgressDetailed(" [done with $key]");
   return ($dest, $width, $height); }
 
